#include <HDL51001_ccf.h>
#include <wdt.h>
#include <uart.h>
#include "util.h"

#define USE_UART 0

extern void gost_encdec(int direction, unsigned int *key, unsigned int *buf, int blocks);

#if 0
void memset(char *buf, char fill, int len);
char *itoa(long cvt, int base);
#endif

#define gost_encode(key,buf,num) gost_encdec(0, key, buf, num)
#define gost_decode(key,buf,num) gost_encdec(1, key, buf, num)

#if USE_UART
    unsigned int _buf[] = { 0x12345678, 0x9abcdef1 };
//    unsigned int _buf[] = { 0x629d1fc2, 0xb61f9aa0 };
#else
    unsigned int _buf[] = { 0x12345678, 0x9abcdef1 };
//    unsigned int _buf[] = { 0x629d1fc2, 0xb61f9aa0 };
#endif

    unsigned int _key[] = {
        0x1eadbeaf,
        0x2eadbeaf,
        0x3eadbeaf,
        0x4eadbeaf,
        0x5eadbeaf,
        0x6eadbeaf,
        0x7eadbeaf,
        0x8eadbeaf
    };

int cmd_pos = 0;
char cmd_buf[32];

static void process_cmd(char *cmd)
{
    int iv;

    if (!*cmd) {
        return;
    } else if (!strcmp(cmd, "encode")) {
        gost_encode(_key, _buf, 1);
    } else if (!strcmp(cmd, "decode")) {
        gost_decode(_key, _buf, 1);
    } else if (!strcmp(cmd, "print")) {
do_print:
        uart_send_str("\r\n{", UART3);
        uart_send_str(itoa(_buf[0], 16), UART3);
        uart_send_str(", ", UART3);
        uart_send_str(itoa(_buf[1], 16), UART3);
        uart_send_str("}\r\n ", UART3);
    } else if (!strcmp(cmd, "enc-print")) {
        gost_encode(_key, _buf, 1);
        goto do_print;
    } else if (!strcmp(cmd, "dec-print")) {
        gost_decode(_key, _buf, 1);
        goto do_print;
    }
}

void cmd_update(void (*cmd_cb)(char *))
{
    char in;
    in = UART_GET_BYTE(UART3);

    switch (in)
    {
//	case '\r':
//		break;
	case '\r':
		if (cmd_pos > 0) {
			memset(cmd_buf+cmd_pos, 0, sizeof(cmd_buf)-cmd_pos);
			cmd_cb(cmd_buf);
		}
		cmd_pos = 0;
		uart_send_str("\r\n# ", UART3);
		break;
	case 8:
	case 0x7f:
		if (cmd_pos > 0) {
			cmd_pos--;
			uart_send_str("\x8 \x8", UART3);
		}
		break;
	default:
//		if (!isprint(in))
//			break;
		UART_SEND_BYTE(in, UART3);
		while(UART_FIFO_TX_EMPTY(UART3) == 0);
		cmd_buf[cmd_pos++] = in;
		if (cmd_pos == sizeof(cmd_buf)-1) {
			uart_send_str("\r\n*** cmd overflow ***\r\n# ", UART3);
			cmd_pos = 0;
		}
		break;
    }
}


void main()
{
#if USE_UART
    int k;
    unsigned short int i = 0;
    UART_InitTypeDef UART_InitStructure;

    GPIOD->DIR = ((uint32_t)0x00007000);
    GPIOD->OUT = ((uint32_t)0x00001000);

    UART_InitStructure.BaudRate = 38400; //set baudrate
    UART_InitStructure.TypeParity = 0x00000000; //parity control type
    UART_InitStructure.Parity = 0x00000000; //enable parity control
    UART_InitStructure.FlowControl = 0x00000000; //enable cts/rts
    UART_InitStructure.Mode = 0x00000003; //rx enable - 1 bit, tx enable - 2 bit (rx + tx en)  
        
    uart_init(UART3, &UART_InitStructure);
    
    WDT_OFF;
    
    GPIOD->BPS = 0xF00; //uart alternative port
    uart_send_str("System is booting...\r\n", UART3);

#if 0
    GPIOD->OUT |= ((uint32_t)0x00002000);
    uart_send_str(itoa(_buf[0], 16), UART3);
    uart_send_str(", ", UART3);
    uart_send_str(itoa(_buf[1], 16), UART3);
    GPIOD->OUT |= ((uint32_t)0x00004000);

    gost_decode(_key, _buf, 1);

    GPIOD->OUT &= ~((uint32_t)0x00004000);

    uart_send_str("enc={", UART3);
    uart_send_str(itoa(_buf[0], 16), UART3);
    uart_send_str(", ", UART3);
    uart_send_str(itoa(_buf[1], 16), UART3);
    uart_send_str("}\r\ndec={", UART3);

    gost_encode(_key, _buf, 1);
    uart_send_str(itoa(_buf[0], 16), UART3);
    uart_send_str(", ", UART3);
    uart_send_str(itoa(_buf[1], 16), UART3);
    uart_send_str("}\r\n ", UART3);
#endif

    uart_send_str("# ", UART3);

    GPIOD->OUT = ((uint32_t)0x00004000);
//    for (i=0;i<100000;i++) ;
    i = 1;
    k = 0;
    while (1) {
#if 0
        if (!(i % 10000)) {
	    k ^= 1;
            i = 0;
            if (k)
              GPIOD->OUT = ((uint32_t)0x00001000);
            else
              GPIOD->OUT = ((uint32_t)0x00002000);
            i++;
        }
        gost_decode(_key, _buf, 1);
break;
#else
        if (UART_NEW_DATA(UART3))
            cmd_update(process_cmd);
#endif
    }

//    gost_encdec(0, 0x12345678, 0x9abcdef1);
//    gost_encdec(0, 0x12345678, 0x9abcdef1);
//    gost_encdec(1, 0x6d4d9aa5, 0x1d3adef9);
#else
    gost_encode(_key, _buf, 1);
    gost_decode(_key, _buf, 1);
    gost_encode(_key, _buf, 1);
    gost_encode(_key, _buf, 1);
    gost_encode(_key, _buf, 1);
//    gost_decode(_key, _buf, 1);
#endif
}

#if 0
static char *_ltostr2(long cvt, int base, char conv, char *a_it)
{
   /*------------------------------------------------------------------------*/
   /* Local Variables                                                        */
   /*------------------------------------------------------------------------*/
   long  quot,
           rem;
       char  *bnum = "0123456789abcdef0123456789ABCDEF";

   /*------------------------------------------------------------------------*/
   /* The number CVT will be converted to a string by taking the remainder   */
   /* of a division of it by its base, and converting it to a character.     */
   /* The number CVT is then set equal to itself divided by its base, and    */
   /* this continues until CVT is 0.                                         */
   /*------------------------------------------------------------------------*/

   if(! cvt) *(a_it--) = '0';

   while(cvt)
   {
      quot = cvt / base;
      rem = cvt - (quot * base);

      if(conv == 'X') rem += 16;
      *(a_it--) = bnum[rem];
      cvt = quot;
   }

   return a_it + 1;
}

char *itoa(long cvt, int base)
{
	static char buf[20];
	char n;

	memset(buf, '0', 15);
	buf[15] = '\0';
	return _ltostr2(cvt, base, base == 10 ? 'd' : 'X', buf+14);
}

void memset(char *buf, char fill, int len)
{
    while (len-- > 0)
    {
       *buf = fill;
       buf++;
    }
}
#endif
